//
// file :         acqthread.h.cpp
//
// description :  thread class
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
// 

#ifndef ACQTHREAD_H
#define ACQTHREAD_H

#include <omnithread.h>
#include "TucsenCam.h"
#include "tucsencam_err_map.h"

#define TUCSENCAM_AUTO_MODE_OFF   0
#define TUCSENCAM_AUTO_MODE_ONCE  1  /* run once (30 shots) */
#define TUCSENCAM_AUTO_MODE_ON    2  /* continuos */

#define TUCSENCAM_MAX_HOR_SIZE 2048
#define TUCSENCAM_MAX_VER_SIZE 2048

#define TUCSENCAM_READ_TOUT 3 // s

typedef short TUCSENERR;
#define TUCSENERR_SUCCESS 1
#define TUCSENERR_ERROR   0


const int tempOffset = 50; // temperature range offset; library maps -50:50C to 0:100

class acqthread :  public omni_thread, public Tango::LogAdapter
{
	
	public:
		/* acqthread::acqthread(): initialize init loop */
		acqthread(TucsenCam_ns::TucsenCam *s):Tango::LogAdapter(s) {myacq = s;};
		/* acqthread::~acqthread(): object destructor */
		~acqthread() {};

		TUCSENERR CameraInitAPI(void);
		TUCSENERR CameraReleaseAPI(void);
			
		TUCSENERR CameraInit();		
		TUCSENERR CameraBufferInit(void);
		TUCSENERR CameraStartAcquisition();
		TUCSENERR CameraSoftTrigger();
		TUCSENERR CameraStopAcquisition();
	
		TUCSENERR CameraInfo();
		TUCSENERR CameraGetTriggerMode(bool force_read, int *nTgrMode, int *nExpMode, int *nEdgeMode, int *nDelayTm, int *nFrames);
		TUCSENERR CameraSetTriggerMode(int *nTgrMode, int *nExpMode, int *nEdgeMode, int *nDelayTm, int *nFrames);
		TUCSENERR CameraEnableHwTrigger(bool hw_trigger);

		TUCSENERR CameraGetTriggerOut(bool force_read, int *nTgrOutPort, int *nTgrOutMode, int *nEdgeMode, int *nDelayTm, int *nWidth);
		TUCSENERR CameraSetTriggerOut(int *nTgrOutPort, int *nTgrOutMode, int *nEdgeMode, int *nDelayTm, int *nWidth);

		TUCSENERR CameraGetROI(bool force_read, bool *bEnable, int *nHOffset, int *nVOffset, int *nWidth, int *nHeight);
		TUCSENERR CameraSetROI(bool *bEnable, int *nHOffset, int *nVOffset, int *nWidth, int *nHeight);
		TUCSENERR CameraResetROI(void);

		TUCSENERR CameraGetExposure(bool force_read, double *exposure);
		TUCSENERR CameraSetExposure(double exposure);

		TUCSENERR CameraGetTemperature(bool force_read, double *temperature);
		TUCSENERR CameraSetTemperature(double temperature);
		TUCSENERR CameraGetTemperatureSetpoint(bool force_read, double *rd_temperature);

		TUCSENERR CameraGetGain(bool force_read, double *gain);
		TUCSENERR CameraSetGain(double gain);

		string CameraGetModel(void);
		string CameraGetInfo(void);
		string CameraGetTriggerInfo(void);
		string CameraGetTriggerOutInfo(void);


		TUCSENERR set_roi();
		void camera_stop();
		void camera_pause();
		void init_timeouts();
		void delete_data(void);

		TUCSENERR init_data(void);
		bool save_data(void *buf, uint32_t curr_frame); 
		void update_configuration(void);
		TUCSENERR update_attributes(void);

		bool auto_gain_feedback();
		bool auto_exposure_feedback();

		void set_sensor_width(int32_t data) {sensor_width = data;}
		void set_sensor_height(int32_t data) {sensor_height = data;}
		void set_width(int32_t data) {width = data;}
		void set_height(int32_t data) {height = data;}
		void set_offset_hor(int32_t data) {offset_hor = data;}
		void set_offset_ver(int32_t data) {offset_ver = data;}

		int32_t get_sensor_width(void) {return sensor_width;}
		int32_t get_sensor_height(void) {return sensor_height;}
		int32_t get_width(void) {return width;}
		int32_t get_height(void) {return height;}
		int32_t get_offset_hor(void) {return offset_hor;}
		int32_t get_offset_ver(void) {return offset_ver;}		


	private:

		// private camera structures from library
		TUCAM_INIT itApi;            // Initializing SDK environment parameters
		TUCAM_OPEN opCam;            // Open camera parameters
		TUCAM_ROI_ATTR roiAttr;      // ROI attribute
		TUCAM_TRIGGER_ATTR tgrAttr;  // trigger attribute
		TUCAM_TRGOUT_ATTR tgrOutAttr;// trigger output attribute
		TUCAM_PROP_ATTR expProp;     // exposure properties
		TUCAM_PROP_ATTR tempProp;    // temperature properties
		TUCAM_PROP_ATTR gainProp;    // global gain properties
		TUCAM_FRAME frame;           // Frame object

		// timestamps used to refresh camera params reading after the reading of the image
		struct timespec roiAttr_time,   
			        tgrAttr_time,   
			        tgrOutAttr_time,
			        exposure_time,   
			        temperature_time, 
			        gain_time,
		                data_refresh; 

		void **tucsen_pFrames;
		char *tucsen_buf;

		TucsenCam_ns::TucsenCam *myacq;

		int32_t camera_get_counter;

		int64_t read_counter;

		// camera size parameters
		int32_t width, height, sensor_width, sensor_height, 
			offset_hor, offset_ver, hor_grid_size, ver_grid_size;

		// camera info
		string usb_version, camera_model, api_version, 
		       firmware_version, serial_number, camera_name;

		// camera main parameters
		double min_exposure, max_exposure, exposure, 
		       min_gain, max_gain, gain, 
		       min_temperature, max_temperature, temperature,
		       temperature_setpoint;

		bool init_camera_flag, open_flag, streamgrabber_flag;

		struct timespec bunch_number_dataready_time;

	protected:
		/* acqthread::run():	start init loop */
		void run(void *);
		
};	/* end class acqthread() */


#endif

/* EOF */
